/*
 * Copyright 2009-2010 Maarten Baert
 * maarten-baert@hotmail.com
 * http://www.maartenbaert.be/
 * 
 * This file is part of ExtremePhysics.
 * 
 * ExtremePhysics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ExtremePhysics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with ExtremePhysics. If not, see <http://www.gnu.org/licenses/>.
 * 
 * File: ep_World_IntegrateVelocity.cpp
 * Integrates the velocity of the bodies in the world.
 */

#include "ExtremePhysics.h"

void ep_World::IntegrateVelocity() {
	
	for(ep_Body *body = first_body; body!=NULL; body = body->next) {
		if(!body->issleeping && !body->isgroundbody) {
			
			body->xvel *= body->damping_factor;
			body->yvel *= body->damping_factor;
			body->rotvel *= body->rotdamping_factor;
			body->xvel += body->gravity_x*timestep;
			body->yvel += body->gravity_y*timestep;
			
			for(ep_Force *force = body->first_force; force!=NULL; force = force->next) {
				if(force->local) {
					body->xvel += ep_transform_x(body->rot_sin, body->rot_cos, force->xforce, force->yforce)*((force->ignoremass)? timestep : body->invmass*timestep);
					body->yvel += ep_transform_y(body->rot_sin, body->rot_cos, force->xforce, force->yforce)*((force->ignoremass)? timestep : body->invmass*timestep);
					body->rotvel += ((force->y-body->yoff)*force->xforce-(force->x-body->xoff)*force->yforce)*body->invinertia*((force->ignoremass)? body->mass*timestep : timestep)
					  +force->torque*body->invinertia*((force->ignoremass)? body->mass*timestep : timestep);
				} else {
					body->xvel += force->xforce*((force->ignoremass)? timestep : body->invmass*timestep);
					body->yvel += force->yforce*((force->ignoremass)? timestep : body->invmass*timestep);
					body->rotvel +=
					  ((force->y-body->yoff)*ep_invtransform_x(body->rot_sin, body->rot_cos, force->xforce, force->yforce)
					  -(force->x-body->xoff)*ep_invtransform_y(body->rot_sin, body->rot_cos, force->xforce, force->yforce))*body->invinertia*((force->ignoremass)? body->mass*timestep : timestep)
					  +force->torque*((force->ignoremass)? timestep : body->invinertia*timestep);
				}
			}
			
		}
	}
	
}

